--- @module glove.gui

local langIndex = 0

local componentTable = {}

local viewTable = {}

local screen = {
	width = 0,
	height = 0,
	scale = 1,
}

local defaultFont = love.graphics.newFont(love.graphics.getWidth() / 60)

local defaultStyle = {
	x = 0,
	y = 0,
	width = 0,
	height = 0,
	border_width = "none",
	border_left = "none",
	border_right = "none",
	border_bottom = "none",
	border_top = "none",
	border_color = {166,165,164, 255},
	border_radius = 0,
	bgcolor = {12, 183, 242, 255},
	color = {255, 255, 255},
}

local init = function()
	screen.width = love.graphics.getWidth()
	screen.height = love.graphics.getHeight()
end

local convertPos = function(coord, axisSize)
	if type(coord) == "string" then
		local value = tonumber(coord)
		if value >= 0 then 
			return value * axisSize / 100
		else
			return axisSize + value * axisSize / 100
		end
	else
		if coord >= 0 then
			return coord
		else
			return axisSize + coord
		end
	end
end

local convertSize = function(size, axisSize)
	local value = 0
	
	if type(size) == "string" then
		value = tonumber(size)
		return value * axisSize / 100
	else
		return size
	end
end

local calcPosition = function(component)
	local x, y = 0, 0
	
	if component.parent then
		x = x + componentTable[component.parent].computed.x
		y = y + componentTable[component.parent].computed.y
	end
	x = x + convertPos(component.style.x, screen.width)
	y = y + convertPos(component.style.y, screen.height)
	return x, y
end

local calcSize = function(style)
	local width = convertSize(style.width, screen.width)
	local height = convertSize(style.height, screen.height)
	return width, height
end

local calcBorders = function(style)
	local l, t, r, b = 0, 0, 0, 0

	if style.border_width ~= "none" then
		local defaultBorder = style.border_width
		l = style.border_left ~= "none" and convertSize(style.border_left, screen.width) or defaultBorder
		t = style.border_top ~= "none" and convertSize(style.border_top, screen.height) or defaultBorder
		r = style.border_right ~= "none" and convertSize(style.border_right, screen.width) or defaultBorder
		b = style.border_bottom ~= "none" and convertSize(style.border_bottom, screen.height) or defaultBorder
	end
	return l, t, r, b
end

local setStyle = function(component, style)
	if type(style) == "table" then
		component.style = style
	end
	for key, value in pairs(defaultStyle) do
		if not component.style[key] then component.style[key] = value end
	end
end

local register = function(component, name)
	if not componentTable[name] then
		component.computed = {}
		component.recompute = true
		componentTable[name] = component
	else
		error('a component named'..name..'already exists.')
	end
	return component
end

local boxCompute = function(box)
	box.computed.x, box.computed.y = calcPosition(box)
	box.computed.width, box.computed.height = calcSize(box.style)
	box.computed.borderL, box.computed.borderT, box.computed.borderR, box.computed.borderB = calcBorders(box.style)
	if box.computed.borderL > box.computed.width then box.computed.borderL = 0 end
	if box.computed.borderT > box.computed.height then box.computed.borderT = 0 end
	if box.computed.borderB > box.computed.height then box.computed.borderB = 0 end
	if box.computed.borderR > box.computed.width then box.computed.borderR = 0 end
end

local boxDraw = function(box)
	local minborder = math.min(box.computed.borderL, box.computed.borderT, box.computed.borderR, box.computed.borderB)
	local maxborder = math.max(box.computed.borderL, box.computed.borderT, box.computed.borderR, box.computed.borderB)
	local radius = box.style.border_radius - minborder / 2

	-- if the radius is too small compared to the border, it will be negative so we fix this manually
	if radius <= 0 then radius = box.style.border_radius / 2 end

	-- draw the inner box
	love.graphics.setColor(box.style.bgcolor)
	love.graphics.rectangle("fill", box.computed.x + minborder / 2, box.computed.y + minborder / 2, box.computed.width - minborder, box.computed.height - minborder, radius, radius)

	-- if there is a border, we change the color to the border's color
	if maxborder > 0 then love.graphics.setColor(box.style.border_color) end

	-- if the smallest border has a width, we set the line width to its size, else we set it to one
	if minborder > 0 then love.graphics.setLineWidth(minborder) else love.graphics.setLineWidth(1) end

	-- draw the outer box
	love.graphics.rectangle("line", box.computed.x + minborder / 2, box.computed.y + minborder / 2, box.computed.width - minborder, box.computed.height - minborder, radius, radius)

	--if the border isn't rounded, we can draw borders with different sizes
	if radius == 0 then
		if box.computed.borderL > minborder then
			love.graphics.rectangle("fill", box.computed.x, box.computed.y, box.computed.borderL, box.computed.height)
		end
		if box.computed.borderT > minborder then
			love.graphics.rectangle("fill", box.computed.x, box.computed.y, box.computed.width, box.computed.borderT)
		end
		if box.computed.borderR > minborder then
			love.graphics.rectangle("fill", box.computed.x + box.computed.width - box.computed.borderR, box.computed.y, box.computed.borderR, box.computed.height)
		end
		if box.computed.borderB > minborder then
			love.graphics.rectangle("fill", box.computed.x, box.computed.y + box.computed.height - box.computed.borderB, box.computed.width, box.computed.borderB)
		end
	end
end
local newBox = function(name, style)
	local box = {}

	box.type = "box"
	box.components = {}
	box.compute = boxCompute
	box.draw = boxDraw
	setStyle(box, style)
	return register(box, name)
end

local buttonCompute = function(button)
	boxCompute(button)
	button.computed.text = button.caption
	button.computed.font = defaultFont
	button.computed.textX = math.floor(button.computed.x + (button.computed.width - button.computed.font:getWidth(button.computed.text)) / 2)
	button.computed.textY = math.floor(button.computed.y + (button.computed.height - button.computed.font:getHeight() * (select(2, button.computed.text:gsub('\n', '\n')) + 1)) / 2)
	button.recompute = false
end

local buttonDraw = function(button)
	boxDraw(button)
	love.graphics.setColor(button.style.color)
	love.graphics.print(button.computed.text, button.computed.textX, button.computed.textY)
end

local newButton = function(name, caption, style)
	local button = {}

	button.type = "button"
	button.caption = caption or ""
	button.compute = buttonCompute
	button.draw = buttonDraw
	setStyle(button, style)
	return register(button, name)
end

local draw = function()
	for _,v in pairs(componentTable) do
		v:compute()
		v:draw()
    end
end

local update = function(dt)
end

return {
	init = init,
	draw = draw,
	update = update,
	newBox = newBox,
	newButton = newButton,
}